/*
Author: Siddharth Salunkhe
Last Update: 20 March 2020
Version: 1.3
**/

// This #include statement was automatically added by the Particle IDE.
#include <ArduinoJson.h>
#include <Arduino.h>
#define GREEN D2
#define RED D1
#define BLUE D0


int weatherID = 0; //stores the weather code for the current weather
float tempF = 0; //stores the current temperature
float pwmGreen = 0; // controls the intensity of the green LED channel
float pwmRed = 0; // controls the intensity of the red LED channel
float pwmBlue = 0; // controls the intensity of the blue LED channel
int flashCyc = 0; // controls if the lights flicker or not
int flickerSpeed = 50; //controls how fast the lights flicker based on precipitation. Stronger rain means faster flicker. No rain means no flicker

// Helper function prototypes
bool parse(const char* json);
bool ctrlLights(bool precip, float temp);
int saturate(float val);



/*
initializes the RGB PWM pins to output pins
initializes the connection between the Photon and the webhook that requests data from the weather site

inputs:
returns:
**/
void setup() {
  // Subscribe to the integration response event
  Particle.subscribe("hook-response/WeatherType", myHandler, MY_DEVICES);
  pinMode(GREEN,OUTPUT);
  pinMode(RED,OUTPUT);
  pinMode(BLUE,OUTPUT);
}

/*
when the weather website returns information, this handler is called to process the information
The parse function is called which takes the raw json data from the website and extracts the weather information

inputs: const char*, const char*
returns:
**/
void myHandler(const char *event, const char *data) {
  // Handle the integration response

    bool l = parse(data);
    // parseTest();
    delay(10000);
}

/*
The main control loop for the Photon. In here we ping the website to trigger a response, which is processed by the myHandler function.
Based on the results of that, the lights in the RGB strip are controlled accordingly.
inputs:
returns:
**/
void loop() {
    String data = String(10);
  // Trigger the integration
  Particle.publish("WeatherType", data, PRIVATE);
  ctrlLights(weatherID,tempF);

    delay(100);
}

/*
Takes in the raw json data, processes it using an imported arduino library and stores the weather data to the weatherID and tempF global variables
inputs:  const char*
returns: bool
**/
bool parse(const char* json){
    const size_t capacity = JSON_ARRAY_SIZE(1) + JSON_OBJECT_SIZE(1) + 2*JSON_OBJECT_SIZE(2) + JSON_OBJECT_SIZE(4) + 2*JSON_OBJECT_SIZE(6) + JSON_OBJECT_SIZE(12) + 270;
    DynamicJsonDocument doc(capacity);

    deserializeJson(doc, json);

    int coord_lon = doc["coord"]["lon"]; // 139
    int coord_lat = doc["coord"]["lat"]; // 35

    JsonObject weather_0 = doc["weather"][0];
    int weather_0_id = weather_0["id"]; // 800
    const char* weather_0_main = weather_0["main"]; // "Clear"
    const char* weather_0_description = weather_0["description"]; // "clear sky"
    const char* weather_0_icon = weather_0["icon"]; // "01n"

    const char* base = doc["base"]; // "stations"

    JsonObject main = doc["main"];
    float main_temp = main["temp"]; // 281.52
    float main_feels_like = main["feels_like"]; // 278.99
    float main_temp_min = main["temp_min"]; // 280.15
    float main_temp_max = main["temp_max"]; // 283.71
    int main_pressure = main["pressure"]; // 1016
    int main_humidity = main["humidity"]; // 93

    float wind_speed = doc["wind"]["speed"]; // 0.47
    float wind_deg = doc["wind"]["deg"]; // 107.538

    int clouds_all = doc["clouds"]["all"]; // 2

    long dt = doc["dt"]; // 1560350192

    JsonObject sys = doc["sys"];
    int sys_type = sys["type"]; // 3
    long sys_id = sys["id"]; // 2019346
    float sys_message = sys["message"]; // 0.0065
    const char* sys_country = sys["country"]; // "JP"
    long sys_sunrise = sys["sunrise"]; // 1560281377
    long sys_sunset = sys["sunset"]; // 1560333478

    long timezone = doc["timezone"]; // 32400
    long id = doc["id"]; // 1851632
    const char* name = doc["name"]; // "Shuzenji"
    int cod = doc["cod"]; // 200 // 200
    const char* idNum = String(weather_0_id);
    const char* tNum = String(main_temp);
    Particle.publish("WeatherData",idNum,PRIVATE);
    Particle.publish("WeatherData",tNum,PRIVATE);
    weatherID = weather_0_id;
    tempF = main_temp;
    flickerSpeed = 40 - (int) (weather_0_id%100);
    if(flickerSpeed < 5){
        flickerSpeed = 5;
    }


    return true;
}

/*
based on the temperature, the code finds a linear interpolation of color to indicate that temperature, blue == cold, red == hot, green == moderate.
based on the weather id and the precipitation, the code sets the flicker speed of the lights.
inputs: float, float
returns: bool
**/
bool ctrlLights(int wid, float temp){
    if(wid < 800){
        if (temp < 20){
            pwmGreen = 0.0;
            pwmBlue = 255.0;
            pwmRed = 0.0;

        } else if (temp < 40){
            pwmGreen = (255.0/20)*temp - 1;
            pwmBlue = 255.0;
            pwmRed = 0.0;

        } else if (temp < 60){
            pwmGreen = 255.0;
            pwmBlue = (-255.0/20)*temp + 3*255;
            pwmRed = 0.0;

        } else if (temp < 80){
            pwmGreen = 255.0;
            pwmBlue = 0.0;
            pwmRed = (255.0/20)*temp - 3*255;

        } else if (temp < 100){
            pwmGreen = (-255.0/20)*temp + 5*255;
            pwmBlue = 0.0;
            pwmRed = 255.0;
        } else {
            pwmGreen = 0.0;
            pwmBlue = 0.0;
            pwmRed = 255.0;
        }
        pwmGreen = pwmGreen*(1-(1/flickerSpeed)*flashCyc);
        pwmBlue = pwmBlue*(1-(1/flickerSpeed)*flashCyc);
        pwmRed = pwmRed*(1-(1/flickerSpeed)*flashCyc);


    } else {
        if (temp < 20){
            pwmGreen = 0.0;
            pwmBlue = 255.0;
            pwmRed = 0.0;

        } else if (temp < 40){
            pwmGreen = (255.0/20)*temp - 1;
            pwmBlue = 255.0;
            pwmRed = 0.0;

        } else if (temp < 60){
            pwmGreen = 255.0;
            pwmBlue = (-255.0/20)*temp + 3*255;
            pwmRed = 0.0;

        } else if (temp < 80){
            pwmGreen = 255.0;
            pwmBlue = 0.0;
            pwmRed = (255.0/20)*temp - 3*255;

        } else if (temp < 100){
            pwmGreen = (-255.0/20)*temp + 5*255;
            pwmBlue = 0.0;
            pwmRed = 255.0;
        } else {
            pwmGreen = 0.0;
            pwmBlue = 0.0;
            pwmRed = 255.0;
        }

    }
    int cmdGreen = saturate(pwmGreen);
    int cmdBlue = saturate(pwmBlue);
    int cmdRed = saturate(pwmRed);
    flashCyc = (flashCyc + 1)%flickerSpeed;
    analogWrite(GREEN,cmdGreen);
    analogWrite(BLUE,cmdBlue);
    analogWrite(RED,cmdRed);

}

/*
takes in a float, makes sure it is bounded between 0 and 255 and converts it to an int
inputs: float
returns: int
**/
int saturate(float val){
    if(val < 0){
        return 0;
    }
    if (val > 255){
        return 255;
    }
    return (int) val;
}


